@@ -1,4 +1,4 @@
-/*	$Id: vilter-attachment.h,v 1.2 2006/02/08 16:37:04 mbalmer Exp $ */
+/*	$Id: vilter-attachment.h,v 1.3 2006/02/19 17:07:46 mbalmer Exp $ */
 
 /*
  * Copyright (c) 2004, 2005, 2006 Marc Balmer <marc@msys.ch>
@@ -16,11 +16,6 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#define TIMEOUT 	60
-#define MAXBUF		1024
-#define MAXSTR		64
-#define MAXTRIES	256
-
 SLIST_HEAD(pat_head, pattern);
 
 struct pattern {
