@@ -1,4 +1,4 @@
-/*	$Id: vilter-clamd.c,v 1.29 2006/02/16 08:50:56 mbalmer Exp $	*/
+/*	$Id: vilter-clamd.c,v 1.32 2006/06/24 10:58:16 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004, 2005 Marc Balmer <marc@msys.ch>
@@ -68,16 +68,15 @@ vilter_envfrom(char *env_from, FILE *fp)
 	 * Write "From <sender>" to the temporary file so that clamd
 	 * recognizes the file as e-mail
 	 */
-
-	if (scantype == SCANTYPE_FILESYSTEM && fp != NULL &&
-	    env_from != NULL)
+	if (scantype == SCANTYPE_FILESYSTEM && fp != NULL && env_from != NULL)
 		fprintf(fp, "From %s\r\n", env_from);
 
 	return 0;
 }
 
 int
-vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
+vilter_scan(SMFICTX *ctx, char *fn, size_t fnlen, char *chroot, char *virus,
+    size_t namelen)
 {
 	int			 fd, datfd;
 	int			 try;
@@ -104,17 +103,21 @@ vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 		if (inet_aton(bind_addr, &bind_sockaddr.sin_addr) == 0) {
 			hostent = gethostbyname(bind_addr);
 			if (!hostent) {
-				syslog(LOG_ERR, "clamd: unknown bind address: %s", bind_addr);
+				syslog(LOG_ERR, "clamd: unknown bind address: "
+				    "%s", bind_addr);
 				close(fd);
 				return SCAN_ERROR;
 			}
 
 			bind_sockaddr.sin_family = hostent->h_addrtype;
-			memcpy(&bind_sockaddr.sin_addr, hostent->h_addr, hostent->h_length);
+			memcpy(&bind_sockaddr.sin_addr, hostent->h_addr,
+			    hostent->h_length);
 		}
 
-		if (bind(fd, (struct sockaddr *)&bind_sockaddr, sizeof(bind_sockaddr))) {
-			syslog(LOG_ERR, "clamd: can't bind to address %s", bind_addr);
+		if (bind(fd, (struct sockaddr *)&bind_sockaddr,
+		    sizeof(bind_sockaddr))) {
+			syslog(LOG_ERR, "clamd: can't bind to address %s",
+			    bind_addr);
 			close(fd);
 			return SCAN_ERROR;
 		}
@@ -133,12 +136,14 @@ vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 			return SCAN_ERROR;
 		}
 		server_sockaddr.sin_family = hostent->h_addrtype;
-		memcpy(&server_sockaddr.sin_addr, hostent->h_addr, hostent->h_length);
+		memcpy(&server_sockaddr.sin_addr, hostent->h_addr,
+		    hostent->h_length);
 	}
 
 	try = 0;
 
-	while (connect(fd, (struct sockaddr *)&server_sockaddr, sizeof(server_sockaddr)) == -1) {
+	while (connect(fd, (struct sockaddr *)&server_sockaddr,
+	    sizeof(server_sockaddr)) == -1) {
 		++try;
 
 		if (try == clamd_tries) {
@@ -164,7 +169,8 @@ vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 	switch (scantype) {
 	case SCANTYPE_FILESYSTEM:
 		if (chroot_scanrealpath && chroot != NULL)
-			snprintf(buf, sizeof(buf), "SCAN %s/%s\r\n", chroot, *fn == '/' ? fn + 1 : fn);
+			snprintf(buf, sizeof(buf), "SCAN %s/%s\r\n", chroot,
+			    *fn == '/' ? fn + 1 : fn);
 		else
 			snprintf(buf, sizeof(buf), "SCAN %s\r\n", fn);
 
@@ -196,22 +202,28 @@ vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 			return SCAN_ERROR;
 		}
 		if (bind_addr != NULL) {
-			bzero((char *)&server_sockaddr, sizeof(server_sockaddr));
+			bzero((char *)&server_sockaddr,
+			    sizeof(server_sockaddr));
 			bind_sockaddr.sin_family = AF_INET;
 
-			if (inet_aton(bind_addr, &bind_sockaddr.sin_addr) == 0) {
+			if (inet_aton(bind_addr, &bind_sockaddr.sin_addr) ==
+			    0) {
 				hostent = gethostbyname(bind_addr);
 				if (!hostent) {
-					syslog(LOG_ERR, "clamd: unknown bind address: %s", bind_addr);
+					syslog(LOG_ERR, "clamd: unknown bind "
+					    "address: %s", bind_addr);
 					close(datfd);
 					return SCAN_ERROR;
 				}
 
 				bind_sockaddr.sin_family = hostent->h_addrtype;
-				memcpy(&bind_sockaddr.sin_addr, hostent->h_addr, hostent->h_length);
+				memcpy(&bind_sockaddr.sin_addr,
+				    hostent->h_addr, hostent->h_length);
 			}
-			if (bind(datfd, (struct sockaddr *)&bind_sockaddr, sizeof(bind_sockaddr))) {
-				syslog(LOG_ERR, "clamd: can't bind to address %s", bind_addr);
+			if (bind(datfd, (struct sockaddr *)&bind_sockaddr,
+			    sizeof(bind_sockaddr))) {
+				syslog(LOG_ERR, "clamd: can't bind to address "
+				    "%s", bind_addr);
 				close(datfd);
 				return SCAN_ERROR;
 			}
@@ -224,19 +236,23 @@ vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 		if (inet_aton(clamd_host, &data_sockaddr.sin_addr) == 0) {
 			hostent = gethostbyname(clamd_host);
 			if (!hostent) {
-				syslog(LOG_ERR, "clamd: unknown host: %s", clamd_host);
+				syslog(LOG_ERR, "clamd: unknown host: %s",
+				    clamd_host);
 				close(datfd);
 				close(fd);
 				return SCAN_ERROR;
 			}
 			data_sockaddr.sin_family = hostent->h_addrtype;
-			memcpy(&data_sockaddr.sin_addr, hostent->h_addr, hostent->h_length);
+			memcpy(&data_sockaddr.sin_addr, hostent->h_addr,
+			    hostent->h_length);
 		} 
 		try = 0;
-		while (connect(datfd, (struct sockaddr *)&data_sockaddr, sizeof(data_sockaddr)) == -1) {
+		while (connect(datfd, (struct sockaddr *)&data_sockaddr,
+		    sizeof(data_sockaddr)) == -1) {
 			++try;
 			if (try == clamd_tries) {
-				syslog(LOG_ERR, "clamd: unable to connect socket");
+				syslog(LOG_ERR, "clamd: unable to connect "
+				    "socket");
 				close(datfd);
 				sclose(sp);
 				return SCAN_ERROR;
@@ -244,12 +260,14 @@ vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 				sleep(1);
 		}
 		if ((fp = fopen(fn, "r")) == NULL) {
-			syslog(LOG_ERR, "clamd: error opening file '%s' for reading", fn);
+			syslog(LOG_ERR, "clamd: error opening file '%s' for "
+			    "reading", fn);
 			close(datfd);
 			sclose(sp);
 			return SCAN_ERROR;
 		}
-		if (to_send(datfd, CLAMD_FROM, sizeof(CLAMD_FROM), 0, t) != sizeof(CLAMD_FROM)) {
+		if (to_send(datfd, CLAMD_FROM, sizeof(CLAMD_FROM), 0, t) !=
+		    sizeof(CLAMD_FROM)) {
 			syslog(LOG_ERR, "clamd: error sending From");
 			close(datfd);
 			sclose(sp);
@@ -269,7 +287,8 @@ vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 		close(datfd);
 		break;
 	default:
-		syslog(LOG_ERR, "clamd: internal error, unknown scantype %d", scantype);
+		syslog(LOG_ERR, "clamd: internal error, unknown scantype %d",
+		    scantype);
 		close(fd);
 		return SCAN_ERROR;
 	}
@@ -300,7 +319,8 @@ vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 			syslog(LOG_ERR, "clamd: error '%s'", buf);
 			return SCAN_ERROR;
 		} else
-			syslog(LOG_ERR, "clamd: unrecognized response: '%s'", p);
+			syslog(LOG_ERR, "clamd: unrecognized response: '%s'",
+			    p);
  	}
 	return SCAN_ERROR;
 }
