@@ -1,4 +1,4 @@
-/*	$Id: imsg.h,v 1.11 2006/02/08 21:30:31 mbalmer Exp $ */
+/*	$Id: imsg.h,v 1.13 2006/03/21 07:01:08 mbalmer Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -82,8 +82,9 @@ enum imsg_type {
 	IMSG_RECONFIGURE,
 #ifdef ENABLE_PF
 	IMSG_PFTABLE_ADD,
-	IMSG_PFTABLE_DEL
+	IMSG_PFTABLE_DEL,
 #endif
+	IMSG_STATS
 };
 
 struct imsg_hdr {
@@ -107,7 +108,20 @@ struct pftable_msg {
 };
 #endif
 
-void		 send_imsg_session(int, pid_t, void *, u_int16_t);
+/*
+ * per connection statistics
+ * if the connection is aborted, msgs is 0
+ */
+struct stats_msg {
+	u_int32_t 	msgs;
+	u_int32_t	aborts;
+	u_int32_t 	spam;
+	u_int32_t 	virus;
+	u_int32_t 	unwanted;
+	u_int32_t	err;
+};
+
+void	 send_imsg_session(int, pid_t, void *, u_int16_t);
 
 /* buffer.c */
 struct buf	*buf_open(size_t);
@@ -128,7 +142,7 @@ int	 imsg_read(struct imsgbuf *);
 int	 imsg_get(struct imsgbuf *, struct imsg *);
 int	 imsg_compose(struct imsgbuf *, enum imsg_type, u_int32_t, pid_t, int,
 	    void *, u_int16_t);
-struct buf	*imsg_create(struct imsgbuf *, enum imsg_type, u_int32_t, pid_t,
+struct buf	*imsg_create(enum imsg_type, u_int32_t, pid_t,
 		    u_int16_t);
 int	 imsg_add(struct buf *, void *, u_int16_t);
 int	 imsg_close(struct imsgbuf *, struct buf *);
