@@ -1,4 +1,4 @@
-/*	$Id: pathnames.h,v 1.5 2006/02/07 19:33:26 mbalmer Exp $	*/
+/*	$Id: pathnames.h,v 1.7 2006/09/17 12:44:23 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2005, 2006 Marc Balmer <marc@msys.ch>
@@ -16,12 +16,7 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#define _PATH_PIDFILE		"/var/run/smtp-vilter.pid"
 #define _PATH_PORT		"unix:/var/run/smtp-vilter.sock"
 #define _PATH_TMPDIR		"/tmp"
 
-#ifdef __FreeBSD__
-#define	_PATH_CFGFILE		"/usr/local/etc/smtp-vilter/smtp-vilter.conf"
-#else
 #define	_PATH_CFGFILE		"/etc/smtp-vilter/smtp-vilter.conf"
-#endif
