@@ -1,4 +1,4 @@
-/*	$Id: smtp-vilter.c,v 1.151 2006/02/17 07:33:14 mbalmer Exp $	*/
+/*	$Id: smtp-vilter.c,v 1.169 2006/06/24 10:44:37 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004, 2005, 2006 Marc Balmer <marc@msys.ch>
@@ -32,7 +32,6 @@
 #ifdef ENABLE_LDAP
 #include <ldap.h>
 #endif
-#include <math.h>
 #include <pthread.h>
 #include <pwd.h>
 #include <signal.h>
@@ -44,9 +43,6 @@
 #include <time.h>
 #include <unistd.h>
 
-#include "libmilter/mfapi.h"
-
-
 #include "pathnames.h"
 #include "smtp-vilter.h"
 
@@ -60,10 +56,6 @@
 #include <sys/syslimits.h>
 #endif
 
-#ifdef __FreeBSD__
-#include "strtonum.h"
-#endif
-
 #define MAX_FILE	1024
 #define MAXLEN		256
 #define MAXSTR		256
@@ -96,7 +88,6 @@ char	*clean_table;
 int	 markall;
 int	 logall;
 int	 verbose;
-char	*pidfile;
 char	*cfgfile;
 char	*tmpdir;
 int	 tmpfiles_gr;
@@ -106,6 +97,9 @@ char	*recipient_notification;
 char	*notify_only;
 int	 log_facility;
 char	*logfile;
+char	*statfile;
+time_t	 stat_begin;
+unsigned int stat_interval;
 FILE	*logfp;
 char	*user;
 char	*group;
@@ -114,9 +108,21 @@ rlim_t	 rlimit_nofile;
 rlim_t	 rlimit_nproc;
 int	 nbe;
 char	*spamprefix;
+char	*spam_headerf;
+char	*spam_headerv;
 int	 logvirus, logspam, logunwanted;
 int	 keep_tmpfiles;
 
+/* statistics */
+
+u_int32_t	n_conn;
+u_int32_t	n_aborts;
+u_int32_t	n_msgs;
+u_int32_t	n_virus;
+u_int32_t	n_spam;
+u_int32_t	n_unwanted;
+u_int32_t	n_err;
+
 struct imsgbuf	*ibuf_e;
 
 extern char *__progname;
@@ -138,6 +144,8 @@ int	 ldap_use_tls;
 volatile sig_atomic_t quit = 0;
 volatile sig_atomic_t child_quit = 0;
 volatile sig_atomic_t reconfig = 0;
+volatile sig_atomic_t dumpstat = 0;
+volatile sig_atomic_t alrm_expired = 0;
 
 struct backend *be;
 int	 dev = -1;
@@ -148,7 +156,7 @@ struct privdata {
 	void			*backend_data;
 };
 
-int dispatch_imsg(struct imsgbuf *, int);
+int dispatch_imsg(struct imsgbuf *);
 
 extern void smtp_vilter_init(void);
 extern int e_main(uid_t, gid_t, int[2]);
@@ -168,6 +176,12 @@ sighdlr(int signum)
 		/* reconfigure */
 		reconfig = 1;
 		break;
+	case SIGALRM:
+		alrm_expired = 1;
+		/* FALLTHROUGH */
+	case SIGUSR1:
+		dumpstat = 1;
+		break;
 	}
 }
 
@@ -179,7 +193,11 @@ static void
 usage(void)
 {
 	fprintf(stderr, "usage: %s"
-	    " [-Vvmkx]"
+	    " [-Vvmkx"
+#ifdef ENABLE_LDAP
+	    "Z[Z]"
+#endif
+	    "]"
 	    " [-A path]"
 	    " [-a spam-subject-prefix]"
 #ifdef ENABLE_LDAP
@@ -191,17 +209,20 @@ usage(void)
 #ifdef ENABLE_LDAP
 	    " [-D binddn]"
 #endif
-	    " [-e logile]"
+	    " [-e logfile]"
 	    " [-f maxfiles]"
 	    " [-g group]"
 #ifdef ENABLE_LDAP
 	    " [-h ldaphost]"
+#endif
+	    " [-i interval]"
+#ifdef ENABLE_LDAP
 	    " [-L ldapport]"
 #endif
 	    " [-n recipient-notification]"
 	    " [-o notify-only]" 
-	    " [-P pidfile]"
  	    " [-p port]"
+	    " [-s statfile]"
 	    " [-T tmpfile-opt]"
 	    " [-t chroot-dir]"
 #ifdef ENABLE_LDAP
@@ -236,9 +257,9 @@ decode_backend(char *s)
 
 	free(q);
 
-	if ((be = (struct backend *)malloc(nbe * sizeof(struct backend))) == NULL)
+	if ((be = (struct backend *)calloc(nbe, sizeof(struct backend))) ==
+	    NULL)
 		err(1, "error allocating memory for backends");
-	bzero(be, nbe * sizeof(struct backend));
 
 	n = 0;
 	do {
@@ -253,7 +274,8 @@ decode_backend(char *s)
 
 #ifdef ENABLE_LDAP
 static int
-get_rebind_credentials(LDAP *ld, char **dnp, char **pwp, int *authmethodp, int freeit)
+get_rebind_credentials(LDAP *ld, char **dnp, char **pwp, int *authmethodp,
+    int freeit)
 {
 	if (verbose)
 		warnx("get_rebind_credentials() called");
@@ -265,7 +287,6 @@ int
 main(int argc, char *argv[])
 {
 	int		 ch;
-	struct stat	 statbuf;
 	char		 libname[MAX_FILE];
 	struct passwd	*passwd;
 	struct group	*grp;
@@ -276,23 +297,23 @@ main(int argc, char *argv[])
 	int		 n;
 	pid_t		 child_pid, pid;
 	char		*fnam;	/* file to analyze */
+	FILE		*fp;
 #ifdef ENABLE_LDAP
 	int		 version;
 #endif
 	int		 xmode;
 	const char	*errstr;
-	long long	 rlimit_max;
+	rlim_t		 rlimit_max;
 	int		 pipe_m2e[2];
 	int		 nfds;
 	int		 flags;
+	time_t		 now;
 
 	/* Set initial values */
-
 	port = NULL;
 	tmpdir = NULL;
 	tmpfiles_gr = 0;
 	tmpfiles_setgrp = 0;
-	pidfile = NULL;
 	user = NULL;
 	group = NULL;
 	chrootdir = NULL;
@@ -310,11 +331,14 @@ main(int argc, char *argv[])
 	unwanted_reaction = REACTION_UNSET;
 	clean_reaction = REACTION_UNSET;
 
-	virus_duration = spam_duration = unwanted_duration = clean_duration = 0L;
+	virus_duration = spam_duration = unwanted_duration = clean_duration =
+	    0L;
 	virus_table = spam_table = unwanted_table = clean_table = NULL;
 
 	log_facility = LOG_UNSET;
 	logfile = NULL;
+	statfile = NULL;
+	stat_interval = 0;
 	recipient_notification = NULL;
 	notify_only = NULL;
 	markall = 0;
@@ -322,6 +346,8 @@ main(int argc, char *argv[])
 	rlimit_nofile = 0;
 	rlimit_nproc = 0;
 	spamprefix = NULL;
+	spam_headerf = NULL;
+	spam_headerv = NULL;
 	logvirus = logspam = logunwanted = 0;
 	keep_tmpfiles = 0;
 #ifdef ENABLE_LDAP
@@ -336,11 +362,12 @@ main(int argc, char *argv[])
 	xmode = 0;
 
 	/* Process the commandline */
-
 #ifdef ENABLE_LDAP	
-	while ((ch = getopt(argc, argv, "A:a:B:b:C:D:d:e:f:g:h:kL:mn:o:P:p:T:t:u:U:Vvw:x?")) != -1) {
+	while ((ch = getopt(argc, argv,
+	    "A:a:B:b:C:D:d:e:f:g:h:i:kL:mn:o:p:s:T:t:u:U:Vvw:xZ?")) != -1) {
 #else
-	while ((ch = getopt(argc, argv, "A:a:b:C:d:e:f:g:kmn:o:P:p:T:t:u:Vvx?")) != -1) {
+	while ((ch = getopt(argc, argv,
+	    "A:a:b:C:d:e:f:g:i:kmn:o:p:T:t:s:u:VvxZ?")) != -1) {
 #endif
 		switch (ch) {
 		case 'A':
@@ -378,9 +405,11 @@ main(int argc, char *argv[])
 				rlimit_max = rl.rlim_max;
 			} else
 				rlimit_max = RLIM_INFINITY;
-			rlimit_nofile = (int)strtonum(optarg, 1LL, rlimit_max, &errstr);
+			rlimit_nofile = (int)strtonum(optarg, 1LL,
+			    (long long)rlimit_max, &errstr);
 			if (errstr)
-				errx(1, "number of files is %s: %s", errstr, optarg);
+				errx(1, "number of files is %s: %s", errstr,
+				    optarg);
 			break; 
 		case 'g':
 			group = optarg;
@@ -390,6 +419,13 @@ main(int argc, char *argv[])
 			ldaphost = optarg;
 			break;
 #endif
+		case 'i':
+			stat_interval = (unsigned int)strtonum(optarg, 1LL,
+			    100000000LL, &errstr);
+			if (errstr)
+				errx(1, "statistics dump interval is %s: %s",
+				    errstr, optarg);
+			break;
 		case 'k':
 			keep_tmpfiles = 1;
 			break;
@@ -397,7 +433,8 @@ main(int argc, char *argv[])
 		case 'L':
 			ldapport = strtonum(optarg, 1, 65535, &errstr);
 			if (errstr)
-				errx(1, "ldap port number is %s: %s", errstr, optarg);
+				errx(1, "ldap port number is %s: %s", errstr,
+				    optarg);
 			break;
 #endif
 		case 'm':
@@ -409,19 +446,19 @@ main(int argc, char *argv[])
 		case 'o':
 			notify_only = optarg;
 			break;
-		case 'P':
-			pidfile = optarg;
-			break;
 		case 'p':
 			port = optarg;
 			break;
+		case 's':
+			statfile = optarg;
+			break;
 		case 'T':
 			if (!strcmp(optarg, "g+r+"))
 				tmpfiles_gr = 1;
 			else if (!strcmp(optarg, "setgrp"))
 				tmpfiles_setgrp = 1;
 			else
-				errx(1, "unknown temp file option %s", optarg);
+				errx(1, "unknown tmp file option %s", optarg);
 			break;
 		case 't':
 			chrootdir = optarg;
@@ -438,14 +475,8 @@ main(int argc, char *argv[])
 			printf("%s %s\n", __progname, VERSION);
 #ifdef ENABLE_LDAP
 			printf("LDAP support enabled\n");
-#endif
-#ifndef __OpenBSD__
-#ifdef ENABLE_PF
-			printf("PF support enabled\n");
-#endif
 #endif
 			exit(0);
-			/* NOTREACHED */
 		case 'v':
 			++verbose;
 			break;
@@ -457,6 +488,20 @@ main(int argc, char *argv[])
 		case 'x':
 			xmode = 1;
 			break;
+#ifdef ENABLE_LDAP
+		case 'Z':
+			switch (ldap_use_tls) {
+			case -1:
+				ldap_use_tls = 1;
+				break;
+			case 1:
+				ldap_use_tls = 2;
+				break;
+			default:
+				usage();
+			}
+			break;
+#endif
 		default:
 			usage();
 		}
@@ -468,20 +513,16 @@ main(int argc, char *argv[])
 	*/
 
 	/* Read config file */
-
 	smtp_vilter_init();
 
 	if (!nbe)
 		errx(1, "no backends defined");
 
 	/* Set default values if some variables are not set */
-
 	if (port == NULL)
 		port = strdup(_PATH_PORT);
 	if (tmpdir == NULL)
 		tmpdir = strdup(_PATH_TMPDIR);
-	if (pidfile == NULL)
-		pidfile = strdup(_PATH_PIDFILE);
 	if (backend == NULL)
 		backend = strdup("clamd");
 	if (virus_strategy == -1)
@@ -511,12 +552,14 @@ main(int argc, char *argv[])
 		printf("\n");
 		printf("user: %s\n", user != NULL ? user : "not set");
 		printf("group: %s\n", group != NULL ? group : "not set");
-		printf("chroot: %s\n", chrootdir != NULL ? chrootdir : "not set");
-		printf("recipient-notification: %s\n", recipient_notification != NULL ? recipient_notification : "not set");
-		printf("spam-subject-prefix: %s\n", spamprefix != NULL ? spamprefix : "not set");
+		printf("chroot: %s\n", chrootdir != NULL ? chrootdir :
+		    "not set");
+		printf("recipient-notification: %s\n", recipient_notification
+		    != NULL ? recipient_notification : "not set");
+		printf("spam-subject-prefix: %s\n", spamprefix != NULL ?
+		    spamprefix : "not set");
 		printf("port: %s\n", port);
 		printf("tmpdir: %s\n", tmpdir);
-		printf("pidfile: %s\n", pidfile);
 		printf("logfile: %s\n", logfile != NULL ? logfile : "not set");
 		if (rlimit_nofile != 0)
 			printf("maxfiles: %lld\n", (long long)rlimit_nofile);
@@ -530,10 +573,23 @@ main(int argc, char *argv[])
 		printf("ldapurl: %s\n", ldapurl != NULL ? ldapurl : "not set");
 		printf("ldaphost: %s\n", ldaphost);
 		printf("ldapport: %s\n", ldapport);
-		printf("use tls: %s\n", ldap_use_tls ? "yes" : "no");
-		printf("searchbase: %s\n", searchbase != NULL ? searchbase : "not set");
+		printf("use tls: ");
+		switch (ldap_use_tls) {
+		case 0:
+			printf("never\n");
+			break;
+		case 1:
+			printf("try\n");
+			break;
+		case 2:
+			printf("always\n");
+			break;
+		}
+		printf("searchbase: %s\n", searchbase != NULL ?
+		    searchbase : "not set");
 		printf("binddn: %s\n", binddn != NULL ? binddn : "not set");
-		printf("bindpasswd: %s\n", bindpasswd != NULL ? bindpasswd : "not set");
+		printf("bindpasswd: %s\n", bindpasswd != NULL ?
+		    bindpasswd : "not set");
 #endif
 		printf("virus reaction: ");
 		switch (virus_reaction) {
@@ -599,77 +655,100 @@ main(int argc, char *argv[])
 			warnx("using LDAP server %s", ldapurl);
 	}
 	version = LDAP_VERSION3;
-	if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version) != LDAP_OPT_SUCCESS)
+	if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version) !=
+	    LDAP_OPT_SUCCESS)
 		errx(1, "Failed to set LDAP version 3 protocol");
 
-	if (ldap_use_tls)
-		if (ldap_start_tls_s(ld, NULL, NULL) != LDAP_SUCCESS)
+	if (ldap_use_tls > 0 && ldap_start_tls_s(ld, NULL, NULL) !=
+	    LDAP_SUCCESS) {
+		if (ldap_use_tls > 1)
 			errx(1, "Failed to start TLS for LDAP");
+		else
+			warnx("Failed to start TLS for LDAP");
+	}
 
 	/*
 	 * Setup LDAP rebind, eventually continue with system defaults if
 	 * the initial bind fails?
  	 */
-
 	if (ldap_simple_bind_s(ld, binddn, bindpasswd) != LDAP_SUCCESS)
 		errx(1, "Failed to bind to directory server as '%s'", binddn);
 
 	ldap_set_rebind_proc(ld, get_rebind_credentials, NULL);
 
 	if (verbose)
-		warnx("Directory server at %s server is up, %susing TLS", ldaphost,
-			ldap_use_tls ? "" : "not ");
+		warnx("Directory server at %s server is up, %susing TLS",
+		    ldaphost, ldap_use_tls ? "" : "not ");
 #endif
-	openlog(__progname, verbose ? LOG_CONS | LOG_NDELAY | LOG_PID | LOG_PERROR : LOG_CONS | LOG_NDELAY | LOG_PID, log_facility);
+	openlog(__progname, verbose ? LOG_CONS | LOG_NDELAY | LOG_PID |
+	    LOG_PERROR : LOG_CONS | LOG_NDELAY | LOG_PID, log_facility);
 
 	/* Load the backends */
-
 	for (n = 0; n < nbe; n++) {
-		snprintf(libname, sizeof(libname), "libvilter-%s.so.%d", be[n].name, MAJOR);
+		snprintf(libname, sizeof(libname), "libvilter-%s.so.%d",
+		    be[n].name, MAJOR);
 
 		if (verbose)
-			warnx("loading backend %s from file %s", be[n].name, libname);
+			warnx("loading backend %s from file %s", be[n].name,
+			    libname);
 			
 		if ((be[n].dlhandle = dlopen(libname, RTLD_LAZY)) == NULL)
-			err(1, "error loading backend %s (%s), %s", be[n].name, libname, dlerror());
+			err(1, "error loading backend %s (%s), %s", be[n].name,
+			    libname, dlerror());
 		
 		be[n].be_init = dlsym(be[n].dlhandle, "vilter_init");
 
-		if ((be[n].be_name = dlsym(be[n].dlhandle, "vilter_name")) == NULL)
-			errx(1, "no function vilter_name in backend %s", be[n].name);
+		if ((be[n].be_name = dlsym(be[n].dlhandle, "vilter_name")) ==
+		    NULL)
+			errx(1, "no function vilter_name in backend %s",
+			    be[n].name);
 
-		be[n].be_envfrom = dlsym(be[n].dlhandle, "vilter_envfrom");	/* Optional */
-		
-		if ((be[n].be_type = dlsym(be[n].dlhandle, "vilter_type")) == NULL)
-			errx(1, "no function vilter_type in backend %s", be[n].name);
+		/* Optional */
+		be[n].be_envfrom = dlsym(be[n].dlhandle, "vilter_envfrom");
+		if ((be[n].be_type = dlsym(be[n].dlhandle, "vilter_type")) ==
+		    NULL)
+			errx(1, "no function vilter_type in backend %s",
+			    be[n].name);
 
 		be[n].type = be[n].be_type();
 
 		if (be[n].type & BE_SCAN_VIRUS)	{
-			if ((be[n].be_scan.be_scan_virus = dlsym(be[n].dlhandle, "vilter_scan")) == NULL)
-				errx(1, "no function vilter_scan in backend %s", be[n].name);
+			if ((be[n].be_scan.be_scan_virus =
+			    dlsym(be[n].dlhandle, "vilter_scan")) == NULL)
+				errx(1, "no function vilter_scan in backend %s",
+				    be[n].name);
 			if (be[n].type & (BE_SCAN_SPAM | BE_SCAN_UNWANTED))
-				errx(1, "illegal backend type combination %s", be[n].name);
+				errx(1, "illegal backend type combination %s",
+				    be[n].name);
 		} else if (be[n].type & BE_SCAN_SPAM) {
-			if ((be[n].be_scan.be_scan_spam = dlsym(be[n].dlhandle, "vilter_scan")) == NULL)
-				errx(1, "no function vilter_scan in backend %s", be[n].name);
+			if ((be[n].be_scan.be_scan_spam = dlsym(be[n].dlhandle,
+			    "vilter_scan")) == NULL)
+				errx(1, "no function vilter_scan in backend %s",
+				    be[n].name);
 			if (be[n].type & BE_SCAN_UNWANTED)
-				errx(1, "illegal backend type combination %s", be[n].name);
+				errx(1, "illegal backend type combination %s",
+				    be[n].name);
 		} else if (be[n].type & BE_SCAN_UNWANTED) {
-			if ((be[n].be_scan.be_scan_unwanted = dlsym(be[n].dlhandle, "vilter_scan")) == NULL)
-				errx(1, "no function vilter_scan in backend %s", be[n].name);
+			if ((be[n].be_scan.be_scan_unwanted =
+			    dlsym(be[n].dlhandle, "vilter_scan")) == NULL)
+				errx(1, "no function vilter_scan in backend %s",
+				    be[n].name);
 		} else
-			errx(1, "backend %s has an unknown vilter type", be[n].name);
+			errx(1, "backend %s has an unknown vilter type",
+			    be[n].name);
 
 		be[n].be_exit = dlsym(be[n].dlhandle, "vilter_exit");
 		if (be[n].be_init != NULL) {
 			if (be[n].be_init(be[n].config_file))
-				errx(1, "error initializing backend %s", be[n].name);
+				errx(1, "error initializing backend %s",
+				    be[n].name);
 		}
 	}
 
-	/* Analye a signe file if requested by the user, else normal operation */
-
+	/*
+	 * Analyze a single file if requested by the user,
+	 * else normal operation
+	 */
 	if (fnam != NULL) {
 		char		 scanfn[PATH_MAX];
 		char		 virus[MAXLEN];
@@ -683,21 +762,32 @@ main(int argc, char *argv[])
 			scanresult = SCAN_OK;
 			if (be[n].type & BE_SCAN_VIRUS) {
 				if (verbose)
-					warnx("scanning %s for viruses using backend %s", fnam, be[n].name);
+					warnx("scanning %s for viruses using "
+					    "backend %s", fnam, be[n].name);
 				strlcpy(scanfn, fnam, sizeof(scanfn));
 				bzero(virus, sizeof(virus));
-				scanresult = be[n].be_scan.be_scan_virus(scanfn, sizeof(scanfn), NULL, virus, sizeof(virus));
+				scanresult = be[n].be_scan.be_scan_virus(NULL,
+				    scanfn, sizeof(scanfn), NULL, virus,
+				    sizeof(virus));
 			} else if (be[n].type & BE_SCAN_SPAM) {
 				if (verbose)
-					warnx("scanning %s for spam using backend %s", fnam, be[n].name);
+					warnx("scanning %s for spam using "
+					    "backend %s", fnam, be[n].name);
 				strlcpy(scanfn, fnam, sizeof(scanfn));
-				scanresult = be[n].be_scan.be_scan_spam(scanfn, sizeof(scanfn), NULL, &score, &threshold);
+				scanresult = be[n].be_scan.be_scan_spam(NULL,
+				    scanfn, sizeof(scanfn), NULL, &score,
+				    &threshold);
 			} else if (be[n].type & BE_SCAN_UNWANTED) {
 				if (verbose)
-					warnx("scanning %s for unwanted content using backend %s", fnam, be[n].name);
+					warnx("scanning %s for unwanted content"
+					    " using backend %s", fnam,
+					    be[n].name);
 				strlcpy(scanfn, fnam, sizeof(scanfn));
 				bzero(reason, sizeof(reason));
-				scanresult = be[n].be_scan.be_scan_unwanted(scanfn, sizeof(scanfn), NULL, reason, sizeof(reason));
+				scanresult =
+				    be[n].be_scan.be_scan_unwanted(NULL,
+				    scanfn, sizeof(scanfn), NULL, reason,
+				    sizeof(reason));
 			}
 			switch (scanresult) {
 			case SCAN_OK:
@@ -712,11 +802,13 @@ main(int argc, char *argv[])
 				exval = 1;
 				break;
 			case SCAN_SPAM:
-				printf("classified as spam, score = %f, threshold %f\n", score, threshold);
+				printf("classified as spam, score = %f, "
+				    "threshold %f\n", score, threshold);
 				exval = 1;
 				break;
 			case SCAN_UNWANTED:
-				printf("unwanted content, reson = %s\n", reason);
+				printf("unwanted content, reson = %s\n",
+				    reason);
 				exval = 1;
 				break;
 			}
@@ -746,16 +838,17 @@ main(int argc, char *argv[])
 		}
 	}
 
-	if (!stat(pidfile, &statbuf))
-		errx(1, "pid file %s exists, another copy running?", pidfile);
-
 	if (verbose == 0 &&  daemon(0, 0))
 		err(1, "can't run as daemon");
 
+	/* initialize statistics */
+	n_conn = n_msgs = n_aborts = 0;
+	n_virus = n_spam = n_unwanted = n_err = 0;
+	time(&stat_begin);
+
 	while (!quit) {
 
 		/* Setup imsg stuff, this is a one-way com only */
-
 		if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, pipe_m2e) == -1)
 			errx(1, "imsg setup failed");
 		if ((flags = fcntl(pipe_m2e[0], F_GETFL, 0)) == -1)
@@ -773,7 +866,6 @@ main(int argc, char *argv[])
 		 * Fork into two processes, one to run privileged, one to do the
 		 * work.
 		 */
-
 		if (verbose)
 			warnx("start unprivileged child process");
 
@@ -781,7 +873,6 @@ main(int argc, char *argv[])
 		child_pid = e_main(uid, gid, pipe_m2e);
 
 		/* We are the privileged process */
-
 #ifndef __linux
 		setproctitle("parent");
 #endif
@@ -790,6 +881,14 @@ main(int argc, char *argv[])
 		signal(SIGINT, sighdlr);
 		signal(SIGTERM, sighdlr);
 		signal(SIGHUP, sighdlr);
+		if (statfile != NULL) {
+			signal(SIGUSR1, sighdlr);
+			if (stat_interval > 0) {
+				signal(SIGALRM, sighdlr);
+				alarm(stat_interval);
+			}
+		} else
+			signal(SIGUSR1, SIG_IGN);
 
 		close(pipe_m2e[1]);
 
@@ -808,23 +907,60 @@ main(int argc, char *argv[])
 				}
 			} else if (nfds > 0 && pfd[0].revents & POLLIN) {
 				/* nfds --; */
-				if (!child_quit && dispatch_imsg(ibuf_e, 0) == -1)
+				if (!child_quit && dispatch_imsg(ibuf_e) == -1)
 					quit = 1;
 			}
 
 #ifdef ENABLE_PF
 			if (pftable_timeout())
-				syslog(LOG_WARNING, "can not timeout pf tables");
+				syslog(LOG_WARNING, "can not timeout pf "
+				    "tables");
 #endif
 			if (child_quit && verbose)
 				warnx("child process terminated");
+
+			if (dumpstat) {
+				if (statfile != NULL) {
+					if (verbose)
+						warnx("dumping statistics");
+					time(&now);
+					if ((fp = fopen(statfile, "a")) !=
+					    NULL) {
+						fprintf(fp, "%u\t%u\t%u\t%u\t"
+						    "%u\t%u\t%u\t%u\t%u\n",	
+						    stat_begin, now, n_conn,
+						    n_aborts, n_msgs,
+						    n_virus, n_spam,
+						    n_unwanted, n_err);
+						n_conn = n_aborts = n_msgs = 0;
+						n_virus = n_spam = n_unwanted =
+						    n_err = 0;
+						stat_begin = now;
+						fclose(fp);
+					} else
+						syslog(LOG_ERR, "can not open "
+						    "'%s' for writing",
+						    statfile);
+					if (alrm_expired && stat_interval) {
+						alrm_expired = 0;
+						alarm(stat_interval);
+					}
+				} else
+					syslog(LOG_WARNING, "received SIGUSR1 "
+					    "but no statfile defined");
+				dumpstat = 0;
+			}
 		}
 
+		if (stat_interval && statfile != NULL)
+			alarm(0);
+
 		if (quit && !child_quit) {	/* XXX is this necessary? */
 			sleep(1);
 			if (!child_quit) {
 				if (verbose)
-					warnx("signalling child process to terminate");
+					warnx("signalling child process to "
+					    "terminate");
 				kill(child_pid, SIGTERM);
 			}
 		}
@@ -845,29 +981,32 @@ main(int argc, char *argv[])
 		free(ibuf_e);
 		close(pipe_m2e[0]);
 
-		if (unlink(pidfile))
-			syslog(LOG_ERR, "can not unlink pidfile %s", pidfile);
-
 		signal(SIGCHLD, SIG_DFL);
 		signal(SIGINT, SIG_DFL);
 		signal(SIGTERM, SIG_DFL);
 		signal(SIGHUP, SIG_DFL);
+		signal(SIGUSR1, SIG_DFL);
+		signal(SIGALRM, SIG_DFL);
+
+		/* Wait one second before restarting the child process */
+		if (!quit)
+			sleep(1);
 	}
 
 	closelog();
 #ifdef ENABLE_LDAP
 	ldap_unbind(ld);
 #endif	
-
 	return 0;
 }
 
 int
-dispatch_imsg(struct imsgbuf *ibuf, int idx)
+dispatch_imsg(struct imsgbuf *ibuf)
 {
 	struct imsg	imsg;
 	int		n;
 	int		rv;
+	struct stats_msg	*stat;
 
 	if ((n = imsg_read(ibuf)) == -1)
 		return -1;
@@ -905,8 +1044,19 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx)
 				rv = 1;
 			}
 			break;
-		break;
 #endif
+		case IMSG_STATS:
+			if (verbose)
+				warnx("received statistics from child process");
+			++n_conn;
+			stat = (struct stats_msg *)imsg.data;
+			n_msgs += stat->msgs;
+			n_aborts += stat->aborts;
+			n_virus += stat->virus;
+			n_spam += stat->spam;
+			n_unwanted += stat->unwanted;
+			n_err += stat->err;
+			break;
 		default:
 			break;
 		}
