@@ -1,4 +1,4 @@
-/*	$Id: smtp-vilter.h,v 1.56 2006/02/17 09:35:14 mbalmer Exp $	*/
+/*	$Id: smtp-vilter.h,v 1.62 2006/06/06 13:13:22 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004, 2005, 2006 Marc Balmer <marc@msys.ch>
@@ -22,22 +22,17 @@
 #include <stdio.h>
 #include <unistd.h>
 
-#define	VERSION	"1.3.2"
+#include "libmilter/mfapi.h"
 
-#define MAJOR	7	/* Major and minor version number */
-#define MINOR	2	/* of shared libs (backends) */
+#define	VERSION	"1.3.3"
+
+#define MAJOR	8	/* Major and minor version number */
+#define MINOR	0	/* of shared libs (backends) */
 
 #if __OpenBSD__
 #define ENABLE_PF
 #endif
 
-#if __FreeBSD__
-#include <sys/param.h>
-#if (__FreeBSD_version >= 600026)
-#define ENABLE_PF
-#endif
-#endif
-
 /*
  *	The following values define reasons why an email might be rejected.
  */
@@ -78,9 +73,12 @@ struct backend {
 	int	 (*be_type)(void);
 	int	 (*be_envfrom)(char *, FILE *);
 	union {
-		int (*be_scan_virus)(char *, size_t, char *, char *, size_t);
-		int (*be_scan_spam)(char *, size_t, char *, double *, double *);
-		int (*be_scan_unwanted)(char *, size_t, char *, char *, size_t);
+		int (*be_scan_virus)(SMFICTX *, char *, size_t, char *,
+		    char *, size_t);
+		int (*be_scan_spam)(SMFICTX *, char *, size_t, char *,
+		    double *, double *);
+		int (*be_scan_unwanted)(SMFICTX *, char *, size_t, char *,
+		    char *, size_t);
 	} be_scan;
 	void	 (*be_exit)(void);
 };
@@ -90,7 +88,7 @@ struct backend {
 typedef struct _SOCK {
 	int	 sock;	
 	char	 buf[SOCK_BUFSIZE];
-	int	 size;
+	ssize_t	 size;
 	char	*pos;
 } SOCK;
 
